import {
  ConflictException,
  ForbiddenException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from '../equipo/entity/equipo.entity';
import { EquipoTipoEntrada } from './entity/equipo-tipo-entrada.entity';
import { Operador } from 'src/operador/entity/operador.entity';
import { TipoEntrada } from '../institucion-tipo-entrada/entity/tipo-entrada.entity';
import { FullInformacionEquipoTipoEntradaView } from './entity/views/full-informacion-equipo-tipo-entrada.view';
import { InformacionEquipoTipoEntradaView } from './entity/views/informacion-equipo-tipo-entrada.view';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionTipoEntradaService } from '../institucion-tipo-entrada/institucion-tipo-entrada.service';

@Injectable()
export class EquipoTipoEntradaService {
  constructor(
    @InjectRepository(EquipoTipoEntrada)
    private repository: Repository<EquipoTipoEntrada>,
    @InjectRepository(FullInformacionEquipoTipoEntradaView)
    private fullInformacionEquipoTipoEntradaView: Repository<FullInformacionEquipoTipoEntradaView>,
    @InjectRepository(InformacionEquipoTipoEntradaView)
    private informacionEquipoTipoEntradaView: Repository<InformacionEquipoTipoEntradaView>,
    @Inject(forwardRef(() => EquipoService))
    private equipoService: EquipoService,
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
  ) {}

  async asignar(id_equipo: number, id_tipo_entrada: number) {
    const equipo = await this.equipoService.findById(id_equipo);
    const tipoEntrada =
      await this.institucionTipoEntradaService.findTipoEntradaById(
        id_tipo_entrada,
      );

    // Verifica que no eixsta un registro con estos ids
    return this.findByEquipoTipoEntrada(equipo, tipoEntrada).then(
      (existeEquipoTipoEntrada) => {
        // Error si existe
        if (existeEquipoTipoEntrada)
          throw new ConflictException(
            'Este tipo de entrada ya fue asignado a este equipo.',
          );
        return this.create(equipo, tipoEntrada);
      },
    );
  }

  create(equipo: Equipo, tipoEntrada: TipoEntrada) {
    // Crear registro
    return this.repository
      .save(this.repository.create({ equipo, tipoEntrada }))
      .then((equipoTipoEntrada) => ({
        message: `Se asignó el tipo de entrada: ${tipoEntrada.tipo_entrada}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoTipoEntrada,
      }));
  }

  delete(operador: Operador, id_equipo_tipo_entrada: number) {
    return this.findById(id_equipo_tipo_entrada)
      .then((equipoTipoEntrada) => {
        // Valida que el equipo pertenezca a la institución del operador
        if (
          operador.institucion.id_institucion !=
          equipoTipoEntrada.equipo.carrito.modulo.institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes modificar la información este equipo porque no pertenece a tu institución.',
          );
        // Eliminar registro
        return this.repository.remove(equipoTipoEntrada);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente el tipo de entrada de este equipo.',
      }));
  }

  findFullInfoAllByIdEquipo(id_equipo: number) {
    return this.fullInformacionEquipoTipoEntradaView
      .find({ where: { id_equipo } })
      .then((infoEquipoTipoEntradas) => {
        const equipoTipoEntradas: EquipoTipoEntrada[] = [];

        for (let i = 0; i < infoEquipoTipoEntradas.length; i++)
          equipoTipoEntradas.push(
            this.repository.create({
              id_equipo_tipo_entrada:
                infoEquipoTipoEntradas[i].id_equipo_tipo_entrada,
              tipoEntrada: {
                id_tipo_entrada: infoEquipoTipoEntradas[i].id_tipo_entrada,
                tipo_entrada: infoEquipoTipoEntradas[i].tipo_entrada,
              },
            }),
          );
        return equipoTipoEntradas;
      });
  }

  findById(id_equipo_tipo_entrada: number) {
    return this.informacionEquipoTipoEntradaView
      .findOne({ where: { id_equipo_tipo_entrada } })
      .then((infoEquipoTipoEntrada) => {
        if (!infoEquipoTipoEntrada)
          throw new ConflictException('No existe este id equipo tipo entrada.');
        return this.repository.create({
          id_equipo_tipo_entrada: infoEquipoTipoEntrada.id_equipo_tipo_entrada,
          equipo: {
            id_equipo: infoEquipoTipoEntrada.id_equipo,
            carrito: {
              id_carrito: infoEquipoTipoEntrada.id_carrito,
              modulo: {
                id_modulo: infoEquipoTipoEntrada.id_modulo,
                institucion: {
                  id_institucion: infoEquipoTipoEntrada.id_institucion,
                },
              },
              tipoCarrito: {
                id_tipo_carrito: infoEquipoTipoEntrada.id_tipo_carrito,
              },
            },
          },
          tipoEntrada: {
            id_tipo_entrada: infoEquipoTipoEntrada.id_tipo_entrada,
          },
        });
      });
  }

  findByEquipoTipoEntrada(equipo: Equipo, tipoEntrada: TipoEntrada) {
    return this.repository.findOne({ where: { equipo, tipoEntrada } });
  }
}
